from loafer.message_translators import AbstractMessageTranslator as LoaferAbstractMessageTranslator
from loafer.message_translators import logger

logger.name = __name__


class AbstractMessageTranslator(LoaferAbstractMessageTranslator):
    pass



from loafer.routes import Route as LoaferRoute
from loafer.routes import logger

from .message_translators import AbstractMessageTranslator
from .providers import AbstractProvider

logger.name = __name__


class DrinkerRoute(LoaferRoute):
    def __init__(
        self,
        provider,
        handler,
        name="default",
        message_translator=None,
        error_handler=None,
    ):
        self.name = name

        if not isinstance(provider, AbstractProvider):
            raise TypeError(f"invalid provider instance: {provider!r}")

        self.provider = provider

        if message_translator:
            if not isinstance(message_translator, AbstractMessageTranslator):
                raise TypeError(f"invalid message translator instance: {message_translator!r}")

        self.message_translator = message_translator

        if error_handler:
            if not callable(error_handler):
                raise TypeError(f"error_handler must be a callable object: {error_handler!r}")

        self._error_handler = error_handler

        if callable(handler):
            self.handler = handler
            self._handler_instance = None
        else:
            self.handler = getattr(handler, "handle", None)
            self._handler_instance = handler

        if not self.handler:
            raise ValueError(
                f"handler must be a callable object or implement `handle` method: {self.handler!r}"
            )



class ProviderError(Exception):
    pass


class ProviderRuntimeError(ProviderError):
    pass


class ConfigurationError(Exception):
    pass


class DrinkerException(Exception):
    pass


class DeleteMessage(DrinkerException):
    pass



from cached_property import cached_property
from loafer.managers import LoaferManager, logger

from .dispatchers import DrinkerDispatcher
from .exceptions import ConfigurationError
from .routes import DrinkerRoute
from .runners import DrinkerRunner

logger.name = __name__


class DrinkerManager(LoaferManager):
    def __init__(self, routes, runner=None, _concurrency_limit=None, _max_threads=None):
        self._concurrency_limit = _concurrency_limit
        if runner is None:
            self.runner = DrinkerRunner(on_stop_callback=self.on_loop__stop, max_workers=_max_threads)
        else:
            self.runner = runner

        self.routes = routes

    @cached_property
    def dispatcher(self):
        if not (self.routes and all(isinstance(r, DrinkerRoute) for r in self.routes)):
            raise ConfigurationError(f"invalid routes to dispatch, routes={self.routes}")

        return DrinkerDispatcher(self.routes, max_jobs=self._concurrency_limit)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydrinker']

package_data = \
{'': ['*']}

install_requires = \
['pydrinker-loafer>=1.0.0,<2.0.0']

extras_require = \
{'gcp': ['pydrinker-gcp>=1.0.0-beta.0,<2.0.0']}

setup_kwargs = {
    'name': 'pydrinker',
    'version': '1.0.0',
    'description': 'The most amazing lib to drink (consume) message from your queue provider of preference',
    'long_description': '# pydrinker\n\n![build on github actions](https://github.com/pydrinker/pydrinker/actions/workflows/test.yml/badge.svg?branch=main)\n\nThis is a abstraction layer between [pydrinker-loafer](https://github.com/pydrinker/pydrinker-loafer) and all pydrinker libs, this project makes possible change thinks of original [olist-loafer](https://github.com/olist/olist-loafer) project, the future of this project is switch all core structure to here and remove pydrinker-loafer;\n\nTo understand more about pydrinker see [org page](https://github.com/pydrinker).\n',
    'author': 'Rafael Henrique da Silva Correia',
    'author_email': 'rafael@abraseucodigo.com.br',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rafaelhenrique/pydrinker',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)
